% This version of impr4 works with red and blue from the short sides of the
% image
%
% This m-file opens a calibration (white) and current image and displays
% holes and asperities with low incidence lighting from the top (red) and
% the bottom (blue), along with histograms and image profiles.
% With respect to impr3 only the asperities close to holes are considered
% and image calibration by a white sample acquisition is addedd.
% The algorithm implemented has user-changeable parameters:
%       - light threshold = mean + "X" std dev of each colour component
%       - distance between asperity and hole

clear all
close all
% ImLightOffName=input('filename of image no light > ')
% ImLightOnName=input('filename of image WITH light > ')
% ImLightOffName='picture 3.jpg'

% USE impr4 with these:
%ImLightOnName = 'Concrete.jpg'; % concrete WORKS FINE
%ImLightOnName = 'picture 8.jpg'; % sandpaper
%ImLightOnName = 'picture 9.jpg'; % holes
%ImLightOnName = 'TileSmallDiag.jpg'; % WORKS FINE
%ImLightOnName = 'PaperDiag.jpg' % Too low asperities
%ImLightOnName = 'TileLargeBack.jpg'

%ImLightOnName = 'ConcrSide.jpg'
%ImLightOnName = 'ConcreteSideBetter.jpg'
%ImLightOnName = 'Concr18mm.jpg'
ImLightOnName = 'LapisBack3.jpg'

% ImLightOff = imread(ImLightOffName);
%ImLightOn = double(imread(ImLightOnName));
ImLightOnTEMP = imread(ImLightOnName);
%ImRedLight = imread('Red.jpg');
%ImBlueLight = imread('Blue.jpg');
%ImWhiteLight = imread('White.jpg');
ImAllLight = 0;
%ImAllLight = imread('All.jpg');
% ImAllLight(:,:,:) = 255; % if no calibration image is available

if ImAllLight ~= 0
% ImLightOff=double(ImLightOff);
ImLightOnTEMP=255+double(ImLightOn)-double(ImAllLight); %Remove the variability between components and in space distribution of light
else
    ImLightOnTEMP=double(ImLightOnTEMP);
end    

for i=1:3
    ImLightOn(:,:,i)=ImLightOnTEMP(:,:,i)';
end

clear ImLightOnTEMP
[Rows, Columns, k]=size(ImLightOn);

%for i=1:Rows
%    for j=1:Columns/3
%    ImDiff(i,j) = single(ImLightOn(i,j))-single(ImLightOff(i,j));

ImDiff(:,:,1) = ImLightOn(:,:,3)-ImLightOn(:,:,2); % B-G
    ImDiff(:,:,2) = ImLightOn(:,:,2)-ImLightOn(:,:,3); % G-B for verification
ImDiff(:,:,3) = ImLightOn(:,:,1)-ImLightOn(:,:,2); % R-G
    ImDiff(:,:,4) = ImLightOn(:,:,2)-ImLightOn(:,:,1); % G-R for verification
    ImDiff(:,:,5) = ImLightOn(:,:,3)-ImLightOn(:,:,1); % B-R
    ImDiff(:,:,6) = ImLightOn(:,:,1)-ImLightOn(:,:,3); % R-B for verification
ImDiff(:,:,7) = ImLightOn(:,:,2)-ImLightOn(:,:,3)-ImLightOn(:,:,1); % G-B-R Holes


%    for i=1:Rows
%        for j=1:Columns
%            if ImDiff(i,j,k) < 0
for k=1:7
    if min(min(ImDiff(:,:,k))) < 0
%        min(min(ImDiff(:,:,k)))
        k
        abs(max(-128, min(min(ImDiff(:,:,k)))))
        ImDiff(:,:,k) = ImDiff(:,:,k) + abs(max(-128, min(min(ImDiff(:,:,k))))) ;
    end
%            end %if
%        end %for j
%    end %for i
end %for k

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
YesPlot = 1 % 0=NO 1=YES
if YesPlot==1

figure
colormap(gray(255))
    subplot(2,2,1)
title('Original image')
    image(uint8(ImLightOn))
    grid
axis image
    subplot(2,2,2)
title('Red component')
    image(uint8(ImLightOn(:,:,1)))
axis image
    subplot(2,2,3)
title('Green component')
    image(uint8(ImLightOn(:,:,2)))
axis image
    subplot(2,2,4)
title('Blue component')
    image(uint8(ImLightOn(:,:,3)))
axis image

    
ImDiff=uint8(ImDiff);
    figure
    colormap(gray(255))
for k=1:7
    subplot(4,2,k)
    image(ImDiff(:,:,k))
    title([k])
    axis image
end % for k

figure
imhist(ImDiff(:,:,1))
title('Histogram of intensity levels for the Blue component')
grid

figure
hold on
title('Vertical image profiles for the Blue component')
plot(ImDiff(:,200,1),'g')
plot(ImDiff(:,300,1))
plot(ImDiff(:,400,1),'r')
legend('x=200','x=300','x=400')
grid

figure
imhist(ImDiff(:,:,3))
title('Histogram of intensity levels for the Red component')
grid

figure
hold on
title('Vertical image profiles for the Red component')
plot(ImDiff(:,200,3),'g')
plot(ImDiff(:,300,3))
plot(ImDiff(:,400,3),'r')
legend('x=200','x=300','x=400')
grid

figure
hold on
title('Horizontal image profiles for the Holes component')
plot(ImDiff(200,:,7),'g')
plot(ImDiff(300,:,7))
plot(ImDiff(400,:,7),'r')
legend('y=200','y=300','y=400')
grid

end % if YesPlot
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% line([0 1200], [mean(ImDiff(:,:,7)) ])

% Looking for the light intensity threshold. Mean plus 1 std deviation of
% each component works fine with all images (blue, red and holes).
% A higher threshold means less more reliable hooking asperities and
% inversely, a lower threshold, means more less reliable asperities
% detected.
% With low roughness a higher threshold is better

ThreshAsperitiesBlue = mean2(double(ImDiff(:,:,1))) + 1 * std2(double(ImDiff(:,:,1)))
ThreshAsperitiesRed = mean2(double(ImDiff(:,:,3))) + 1 * std2(double(ImDiff(:,:,3)))
ThreshHoles = mean2(ImDiff(:,:,7)) + 1 * std2(double(ImDiff(:,:,7)))


% imhist(ImDiff(:,:,2)) % distribution of grays in Asperities

% Building a binary mask for the three components
for i=1:Rows
    for j=1:Columns
    if ImDiff(i,j,1) > ThreshAsperitiesBlue % ThreshAsperities BLUE % 150 % 
        ImDiff(i,j,8) = 255; % Hooking areas
    end % if
    if ImDiff(i,j,3) > ThreshAsperitiesRed % ThreshAsperities RED % 150 % 
        ImDiff(i,j,9) = 255; % Hooking areas
    end % if
    if ImDiff(i,j,7) > ThreshHoles
        ImDiff(i,j,10) = 255; % Hole areas
    end % if
end %for j
end %for i

figure
image(ImDiff(:,:,8))
colormap(hot)
title('Blue asperities')
    axis image

figure
image(ImDiff(:,:,9))
colormap(hot)
title('Red asperities')
    axis image
    
figure
image(ImDiff(:,:,10))
colormap(hot)
title('Holes')
    axis image
    
figure
image(uint8(ImLightOn))
title('Original image')
axis image
grid

YesPlot = 1;
if YesPlot == 1

% Finding the hole areas preceeding the asperity (scanning vertically ONLY)

% The distance between asperity and hole should be greater than 1 pixel
% because of possible shade areas.
% BLUE LOOP
bdist = 2; % distance in pixels between asperity and hole (on the image)
for i=1:Rows
    for j=1:Columns
        if ImDiff(i,j,8) == 255
            k = bdist; 
            while [i+k < Rows] & [ImDiff(i+k,j,10) == 255]
                k = k + 1;
            end % while
            if k > bdist % is current asperity pixel close to hole?
            ImBlueLight(i,j,:) = [255 0 255]; % only enhancing the asperity area close to the hole
            end
            ImBlueLight( i+bdist:i+k-1 ,j,1) = 255;
            ImBlueLight( i+bdist:i+k-1 ,j,2) = 255;
            ImBlueLight( i+bdist:i+k-1 ,j,3) = 0;
        end % if
    end %for j
end %for i
end

% RED LOOP
rdist = 2; % distance in pixels between asperity and hole (on the image)
for i=Rows:-1:rdist
    for j=1:Columns
        if ImDiff(i,j,9) == 255
            k = rdist;
            while [i-k > 0] & [ImDiff(i-k,j,10) == 255]
                k = k + 1;
            end % while
            if k > rdist % is current asperity pixel close to hole?
            ImRedLight(i,j,:) = [255 0 255]; % only enhancing the asperity area close to the hole
            end
            ImRedLight( i-k+1:i-rdist ,j,1) = 255;
            ImRedLight( i-k+1:i-rdist ,j,2) = 255;
            ImRedLight( i-k+1:i-rdist ,j,3) = 0;
        end % if
    end %for j
end %for i

figure
image(uint8(ImBlueLight))
title('Blue asperities and preceeding holes')
axis image
grid

figure
image(uint8(ImRedLight))
title('Red asperities and preceeding holes')
axis image
grid

% for additional statistics
impr5